/**
 * \file	GText.h
 * \author	Krzysztof Kaczor kinio4444@gmail.com
 * \date 	15.05.2007 
 * \version	1.15
 * \brief	This file contains class definition that is text drawing on the canvas.
 * \note This file is a part of HQEd. It has been develped within the HeKatE Project.
 * HQEd is a free software that you can redistribute and/or modify
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License or 
 * (at your option) any later version.
 * You should have received a copy of the GNU General Public License with HQEd.
 * \note HQEd is distributed in hope that will be useful.
 * It has NO ANY WARRANTY, neither implied warranty of
 * MERCHANTABILITY nor FITNESS FOR A PARTICULAR PURPOSE. 
 * \note Copyright (C) 2006-9 by the HeKatE Project.
 * \see GNU General Public License for more details http://www.gnu.org/licenses/
 * \see HeKatE project http://hekate.ia.agh.edu.pl
 */

#ifndef GTextH
#define GTextH
//---------------------------------------------------------------------------

#include <QObject>
#include <QGraphicsItem>
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QStyleOptionGraphicsItem>
//--------------------------------------------------------------------------

class XTT_Table;
class ExpressionEditor_ui;
//---------------------------------------------------------------------------
/**
* \class 	GText
* \author	Krzysztof Kaczor kinio4444@gmail.com
* \date 	15.05.2007 
* \brief 	Class definition that is text drawing on the canvas.
*/
class GText : public QObject, public QGraphicsItem
{
	Q_OBJECT

private:

     /// Tabela rodzic
     XTT_Table* fParent;	///< Pointer to the parent of the table.

     /// Wiersz
     unsigned int fRow;	///< Number of the row.

     /// Kolumna
     unsigned int fCol;	///< Number of the column.

     /// Okresla co nalezy pobrac z tabeli, jaki text
     /// 0 - zawartosc komorki okreslonej przez fRow, fCol
     /// 1 - Title
     /// 2 - Header okreslony przez fCol
	/// Determines what type of text should be taken.
	/// \li 0 - content of the cell.
	/// \li 1 - Title.
	/// \li 2 - Header.	
     unsigned int fTextType;
	
	/// Okresla czy komorka ma zostac zaznaczona
	bool isSelected;	///< Determines whether cell should be selected.
     
     bool fIsHeaderSelected;		///< Determines selection of the header cell
	
	/// Polecenia menu kontekstowego
	QAction* aEidtTable;		///< Pointer to the class provides an abstract user interface action.
	QAction* aCellEditor;		///< Pointer to the class provides an abstract user interface action.
	QAction* aEditCell;			///< Pointer to the class provides an abstract user interface action.
	QAction* aDeleteTable;		///< Pointer to the class provides an abstract user interface action.
	QAction* aInsertRowBefore;	///< Pointer to the class provides an abstract user interface action.
	QAction* aInsertRowAfter;	///< Pointer to the class provides an abstract user interface action.
	QAction* aDeleteRow;		///< Pointer to the class provides an abstract user interface action.
	QAction* aDeleteCol;		///< Pointer to the class provides an abstract user interface action.
	QAction* aMoveRowDown;		///< Pointer to the class provides an abstract user interface action.
	QAction* aMoveRowUp;		///< Pointer to the class provides an abstract user interface action.
	QAction* aAttributeEditor;	///< Pointer to the class provides an abstract user interface action.
     
     ExpressionEditor_ui* exprEditor;   ///< Pointer to the expression editor dialog
	
	/// Metoda tworzaca polenia menu
	/// \brief Function creates menu actions.
	///
	/// \return No return value.
	void createActions(void);
	
protected:

	/// \brief Function is called when mouse right button is pressed.
	///
	/// \param event Pointer to event object.
	/// \return No return values.
	void contextMenuEvent(QGraphicsSceneContextMenuEvent* contextMenuEvent);
	
	/// \brief Function is called mouse is hanging over the cell.
	///
	/// \param event Pointer to event object.
	/// \return No return values.
	void hoverEnterEvent(QGraphicsSceneHoverEvent* event);
	
	/// \brief Function is called when mouse is leavin the cell.
	///
	/// \param event Pointer to event object.
	/// \return No return values.
	void hoverLeaveEvent(QGraphicsSceneHoverEvent* event);

public:

	/// \brief Constructor GText class.
	///
	/// \param _parent Pointer to the parent of the table.
	/// \param _tt Index of the table.
	/// \param _row Index of the row of the cell.
	/// \param _col Index of the column of the cell.
	GText(XTT_Table* _parent, unsigned int _tt, unsigned int _row, unsigned int _col);
     
     /// \brief Destructor of GText class.
     ~GText(void);

	/// \brief Function draws square on the screen.
	///
	/// \param _p Pointer to the canvas.
	/// \param _sogi Pointer to the drawing style.
	/// \param widget Unused pointer.
	/// \return No return value.
	void paint(QPainter* _p, const QStyleOptionGraphicsItem* _sogi, QWidget* widget=0);

	/// \brief Function retrieves rectangular in which object is located.
	///
	/// \return Rectangular in which object is located.
	QRectF boundingRect() const;

	/// \brief Function sets number of the row.
	///
	/// \return No return value.
	void setRow(unsigned int _r){ fRow = _r; }

	/// \brief Function sets number of the column.
	///
	/// \return No return value.
	void setCol(unsigned int _c){ fCol = _c; }

	/// \brief Function sets type of the text.
	///
	/// \return No return value.
	void setTextType(unsigned int _tt){ fTextType = _tt; }
     
     /// \brief Allow for select header cell
	///
	/// \param bool _selStat - selection status
	/// \return No return value
	void selectHeaderCell(bool _selStat);
     
     /// \brief Return status of header selection
	///
	/// \return True is header is selected, otherwise false
	bool isHeaderSelected(void);
	
public slots:

	/// \brief Function shows cell contents window.
	///
	/// \return No return value.
	void editCell(void);
     
     /// \brief Function called when cell editor is closed using the OK button
	///
	/// \return No return value.
	void cellEditorConfirm(void);
     
     /// \brief Function called on cell editor close event
	///
	/// \return No return value.
	void cellEditorClose(void);

	/// \brief Function moves one row up.
	///
	/// \return No return value.
	void moveRowUp(void);

	/// \brief Function moves one row down.
	///
	/// \return No return value.
	void moveRowDown(void);

	/// \brief Function inserts one row before another.
	///
	/// \return No return value.
	void insertRowBefore(void);

	/// \brief Function inserts one row after another.
	///
	/// \return No return value.
	void insertRowAfter(void);

	/// \brief Function removes current row.
	///
	/// \return No return value.
	void deleteRow(void);

	/// \brief Function removes current column.
	///
	/// \return No return value.
	void deleteCol(void);

	/// \brief Function shows edit attribute window.
	///
	/// \return No return value.
	void editAttribute(void);
};
//---------------------------------------------------------------------------
#endif
